# cornell.py
# adapted from Fall 2019 Prelim 2 Question 4
# coded by I.Huang during the CS 1110 Sp 2022 Classes Review Session


# We are making Cornell!
# For the purposes of this example, students can only have ONE major and once
# they declare it, they cannot change it (what a scary idea...)

class Cornellian(object):
    """
    A class representing a person at Cornell.

    CLASS ATTRIBUTES:
        NEXT_NUM [int > 0]: a unique number that is assigned to one student
            or professor; it is incremented by 1 every time a Cornellian
            object is created

    INSTANCE ATTRIBUTES:
        first_name [nonempty string]: first name of this Cornellian
        last_name [nonempty string]: last name of this Cornellian
        cuid [string]: unique ID of this Cornellian
    """

    NEXT_NUM = 1

    def __init__(self, first, last):
        """ Initializes a Cornellian with first_name 'first' and last_name 'last'.

        The cuid of this Cornellian is a string with 3 character: the lower-cased first
        letter of the Cornellian's first name and the lower-cased first letter of their
        last name and the value of NEXT_NUM.
        It also increments the class attribute NEXT_NUM by one.

        Precondition:
            first is a nonempty string
            last is a nonempty string
        """
        self.first_name = first
        self.last_name = last
        self.cuid = first[0].lower() + last[0].lower() + str(Cornellian.NEXT_NUM)
        Cornellian.NEXT_NUM += 1


    def __eq__(self, other):
        """
        Returns True if other is a Cornellian object equal to this one.

        Two Cornellians are equal if they:
            1. Both have the same cuid
        NOTICE: Two Cornellians can be equal if they have different names

        Preconditions: None (other can be anything)
        """
        if isinstance(other, Cornellian):
            if self.cuid == other.cuid:
                return True
            else:
                return False
            #return self.cuid == other.cuid
        return False

class Professor(Cornellian):
    """
    A class representing a Professor at Cornell.

    CLASS ATTRIBUTES:
        departments: a list of departments that have Professors
        all_of_em: a list of all Professors

    INSTANCE ATTRIBUTES (those of Cornellian as well as):
        department [nonempty string]: the department of this professor
        advisees [possibly empty list of Student objects]: the Students that
            this professor is an advisor for
    """

    departments = []
    all_of_em = []

    def __init__(self, f, l, d):
        """ Initializes a Professor with first_name 'f' and last_name 'l'.

        The cuid of this Professor is a string with 3 characters: the lower-cased first
        letter of the Professor's first name and the lower-cased first letter of their
        last and then the value of NEXT_NUM.
        It also increments the Cornellian class attribute NEXT_NUM by one.

        Also, sets this Professor's departmnet to 'd' and advisees to the empty
        list (because when a professor first joins Cornell, they aren't assigned
        advisees right away!).

        If this professor's department is not already in Professor.departments,
            this method adds it so the list accurtately represents at
            departments at Cornell.

        Precondition:
            f is a nonempty string
            l is a nonempty string
            d is a (nonempty) string
        """
        super().__init__(f, l)
        self.department = d
        self.advisees = []

        #if Professor.departments.count(d) == 0:
        if d not in Professor.departments:
            Professor.departments.append(d)

        Professor.all_of_em.append(self)


class Student(Cornellian):
    """
    A class representing a student at Cornell.

    INSTANCE ATTRIBUTES (those of Cornellian as well as):
        college [possibly empty string]: the college of this student, empty
            string if unknown
        major [possibly empty string]: the major of this student
            Note: a student may only have one major and once it is declared,
            it may not change
        advisor [Professor or None]: the faculty adivsor of this student; department
            of this Professor should match this Student's major or be None.

    """


    def __init__(self, f, l, college = ""):
        """ Initializes a Student with first_name 'f' and last_name 'l'.

        The cuid of this Students is a string with 3 characters: the lower-cased first
        letter of the Student's first name and the lower-cased first letter of their
        last and then the value of NEXT_NUM.
        It also increments the Cornellian class attribute NEXT_NUM by one.


        Also, sets this Student's college to 'college', let's just say it's optional
        that is default to None.

        Precondition:
            f is a nonempty string
            l is a nonempty string
            college is a (possibly empty) string
                if a college is not given, it is set to the empty string
        """
        super().__init__(f, l)
        self.college = college

    def chooseMajor(self, major):
        """ Updates this Students major to be 'major.'

        Once they pick a major, an advisor is assigned to them. The fauculty
        advisor's department should match the student's major. If there
        is not a professor in the department that matches the Student's major
        (meaning their department is not represent in Professor.departments),
        the Student's advisor is None. If there is more than one professor
        in this department, pick the first professor as they appear in
        Professor.all_of_em.

        Also puts this Student in their advisor's list of advisees if
        the Professor is not None.

        Preconditions:
            major is a string
        """
        self.major = major

        prof = None

        count = 0
        for p in Professor.all_of_em:
            if p.department == major and count == 0:
                prof = p
                count += 1

        self.advisor = prof

        if prof is not None:
            prof.advisees.append(self)
